/* NAME:
 *	proxy_connect - connect a socket to server
 *
 * SYNOPSIS:
 *	const char *use_proxy(const char *"dest", const char *"prefer")
 *	int proxy_connect(int "s", const char *"dest", char **"err")
 *
 * DESCRIPTION:
 *	These routines provide the facility to use the HTTP CONNECT
 *	method to read a "dest" beyond a firewall.
 *
 *	use_proxy() is called with the destination "dest" and an
 *	indication of a prefered proxy "prefer".  The environment will
 *	then be scanned for "prefer_proxy," "prefer", "https_proxy"
 *	and "http_proxy". The first one found in the environment will
 *	be used.
 *
 *	use_proxy() then looks for "no_proxy" and if set, it should
 *	contain a ',' separated list of domains which should be
 *	contacted directly.  Each domain can be one of:
 *
 *	"domain"
 *		an exact match (ignoring case) is required.
 *
 *	".domain"
 *		a tail match on ".domain".
 *
 *	"*.domain"
 *		same as ".domain".
 *
 *	"re"
 *		a POSIX regular expression.  This is not supported on
 *		all platforms, so is best avoided.
 *
 *	'ipaddr'
 *		If "dest" is a dotted IP address.
 *
 *	See domaincmp(3) for more details about the matching process.
 *
 *	proxy_connect() should be passed a socket(2) descriptor "s" which
 *	has been connected to a HTTP proxy and the desired destination
 *	"dest".	 It will handle the CONNECT protocol with the proxy,
 *	and if there is an error and "err" is not null it is set to
 *	point to a copy of the error message from the proxy.
 *
 * RETURNS:
 *	use_proxy() returns NULL if "dest" matches any of the
 *	"domains" specified in "no_proxy", otherwise the "proxy" URL
 *	found in the environment is returned.
 *
 *	proxy_connect() returns the socket "s" connected to "dest"
 *	or -1 on failure.
 *
 * SEE ALSO:
 *	domaincmp(3), net_open(3).
 *	
 * AUTHOR:
 *	Simon Gerraty <sjg@quick.com.au>
 */
/*
 *	@(#)Copyright (c) 1997, Simon J. Gerraty.
 *	
 *	This is free software.	It comes with NO WARRANTY.
 *	Permission to use, modify and distribute this source code 
 *	is granted subject to the following conditions.
 *	1/ that the above copyright notice and this notice 
 *	are preserved in all copies and that due credit be given 
 *	to the author.	
 *	2/ that any changes to this code are clearly commented 
 *	as such so that the author does not get blamed for bugs 
 *	other than his own.
 *	
 *	Please send copies of changes and bug-fixes to:
 *	sjg@quick.com.au
 */

#ifndef lint
static char RCSid[] = "$Id: proxy_connect.c,v 1.10 1998/11/11 06:03:09 sjg Exp $";
#endif
#include <stdio.h>
#ifdef __STDC__
#include <stdlib.h>
#endif
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <string.h>

#include "tcp.h"

extern int errno;

const char *proxy_auth_token = 0;		/* base64 encoded auth token */

/*
 * Use the HTTP CONNECT protocol to connect "s" to "dest"
 */
int	
proxy_connect(s, dest, err)
     int s;
     const char *dest;
     char **err;
{
	static int once = 1;
	static char buf[1024];			/* 512+ msg not unusual */
	char *cp;
	int n, x, sts = -1;

	if (cp = strstr(dest, "://"))
		dest = &cp[3];

	if (once) {
		once = 0;
		if (!proxy_auth_token)
			proxy_auth_token = getenv("proxy_auth_token");
		if (proxy_auth_token && *proxy_auth_token == '\0')
			proxy_auth_token = 0;
	}
	
	if (proxy_auth_token)			/* expected to be base64 */
		(void) sprintf(buf, "CONNECT %s HTTP/1.0\nProxy-authorization: Basic %.20s\r\n\r\n", dest, proxy_auth_token);
	else
		(void) sprintf(buf, "CONNECT %s HTTP/1.0\n\r\n", dest);
	if (write(s, buf, strlen(buf)) > 0) {
		if ((n = read(s, buf, sizeof(buf))) > 4) {
			if (cp = strchr(buf, ' ')) {
				while (*cp == ' ')
					cp++;
				sts = atoi(cp);
				if (sts == 200)
					return s;
				sts = -sts;
				if (err) {
					x = n;
					while ((n = read(s, &buf[x], sizeof(buf) - x)) > 0) {
						x += n;
					}
					buf[x] = '\0';
					*err = buf;
				}
			}
		}
	}
	close(s);
	return (sts);
}

const char *
use_proxy(dest, prefer)
     const char *dest, *prefer;
{
	static int once = 1;
	static char *proxies[] = {
		0, 0,
		"https_proxy",
		"http_proxy",
		0,
	};
	static char **no_proxy = 0, *proxy = 0;
	char buf[256], **p, *cp, *cp2;
	int n, i;

	if ((cp = strchr(dest, '.')) == 0)
		return 0;	/* only proxy for fqdn */

	if (once) {
		once = 0;

		if (prefer) {
			sprintf(buf,  "%s_proxy", prefer);
			proxies[0] = buf;
			proxies[1] = (char *)prefer,
			p = proxies;
		} else
			p = &proxies[2];
		for ( ; proxy == 0 && *p != 0; p++) {
			proxy = getenv(*p);
			if (proxy && *proxy == '\0')
				proxy = 0;
		}
		if (proxy) {
			if (strcasecmp(proxy, "none") == 0) {
				proxy = 0;
			} else {
				cp2 = getenv("no_proxy");
				if (cp2 && *cp2) {
					no_proxy = s2av(strdup(cp2), ",;", &n);
				}
			}
		}
	}
	
	if (no_proxy) {
		/*
		 * we only get here if proxy is set as well as no_proxy
		 */
		while (cp > dest && *cp != '/' && *cp != ':' && *cp != '!')
			cp--;
		if (cp > dest) {
			strcpy(buf, &cp[1]);
		} else {
			strcpy(buf, dest);
		}
		if (cp = strpbrk(buf, ":!"))
			*cp = '\0';
		for (p = no_proxy; *p != 0; p++) {
			if (strcasecmp(*p, "ipaddr") == 0) {
				unsigned long ip = inet_addr(buf);

				if ((long) ip != -1)
					return 0;
			}
			if (domaincmp(buf, *p, 1) == 0)
				return 0;
		}
	}
	return proxy;
}

	  
#ifdef MAIN
#include <signal.h>

int
main(argc, argv)
     int argc;
     char **argv;
{
	const char *proxy;
	char *err;
	int s, x, n, pid;
	char buf[256];

	if (proxy = strrchr(argv[0], '/'))
		proxy++;
	else
		proxy = argv[0];
	
	if (argc < 2) {
		fprintf(stderr, "%s host:port\n", argv[0]);
		exit(1);
	}

	if (proxy = use_proxy(argv[1], (argc > 2) ? argv[2] : proxy)) {
		if ((s = net_open(proxy, "c")) < 0)
			exit(1);
		if ((s = proxy_connect(s, argv[1], &err)) < 0) {
			write(2, err, strlen(err));
			exit(2);
		}
	} else {
		if ((s = net_open(argv[1], "c")) < 0)
			exit(1);
	}
#ifdef TEST
	write(s, "helo\r\n", 6);
	shutdown(s, 1);
	while ((n = read(s, buf, sizeof(buf))) > 0) {
		write(1, buf, n);
	}
	close(s);
	exit(0);
#else
	switch (pid = fork()) {
	case -1:
		perror("fork");
		exit(1);
	case 0:
		n = copy(0, s);
		shutdown(s, 1);
		break;
	default:
		signal(SIGCHLD, SIG_IGN);
		n = copy(s, 1);
		if (errno)
			perror("copy");
		kill(pid, SIGHUP);
		break;
	}
	exit(n);
#endif
}
#endif
